"""
    作者：mldsh
    日期：2022年09月22日14:01
    使用工具：PyCharm
    @File：word_filter.py
    @desc：敏感词过滤
"""


from collections import defaultdict
import re
import os
from pathlib import Path

BASE_DIR = Path(__file__).parent

WordFilePath = os.path.join(BASE_DIR, 'wd.txt')


class NaiveFilter():
    '''Filter Messages from keywords

    very simple filter implementation

    # >>> f = NaiveFilter()
    # >>> f.add("sexy")
    # >>> f.filter("hello sexy baby")
    hello **** baby
    '''

    def __init__(self):
        self.keywords = set([])

    def parse(self, path):
        for keyword in open(path):
            self.keywords.add(keyword.strip().decode('utf-8').lower())

    def word_replace(self, message, repl="*"):
        message = str(message).lower()
        for kw in self.keywords:
            message = message.replace(kw, repl)
        return message


class BSFilter:
    '''Filter Messages from keywords

    Use Back Sorted Mapping to reduce replacement times

    # >>> f = BSFilter()
    # >>> f.add("sexy")
    # >>> f.filter("hello sexy baby")
    hello **** baby
    '''

    def __init__(self):
        self.keywords = []
        self.kwsets = set([])
        self.bsdict = defaultdict(set)
        self.pat_en = re.compile(r'^[0-9a-zA-Z]+$')  # english phrase or not
        self.path = WordFilePath

    def add(self, keyword):
        if isinstance(keyword, bytes):
            keyword = keyword.decode('utf-8')
        keyword = keyword.lower()
        if keyword not in self.kwsets:
            self.keywords.append(keyword)
            self.kwsets.add(keyword)
            index = len(self.keywords) - 1
            for word in keyword.split():
                if self.pat_en.search(word):
                    self.bsdict[word].add(index)
                else:
                    for char in word:
                        self.bsdict[char].add(index)

    def parse(self):
        with open(self.path, "r", encoding='utf-8') as f:
            for keyword in f:
                self.add(keyword.strip())

    def word_replace(self, message, repl="*"):
        if isinstance(message, bytes):
            message = message.decode('utf-8')
        message = message.lower()
        for word in message.split():
            if self.pat_en.search(word):
                for index in self.bsdict[word]:
                    message = message.replace(self.keywords[index], repl)
            else:
                for char in word:
                    for index in self.bsdict[char]:
                        message = message.replace(self.keywords[index], repl)
        return message


class DFAFilter():
    '''Filter Messages from keywords

    Use DFA to keep algorithm perform constantly

    # >>> f = DFAFilter()
    # >>> f.add("sexy")
    # >>> f.filter("hello sexy baby")
    hello **** baby
    '''

    def __init__(self):
        self.keyword_chains = {}
        self.delimit = '\x00'
        self.path = WordFilePath

        # 初始化词库
        self.parse()

    def add(self, keyword):
        if isinstance(keyword, bytes):
            keyword = keyword.decode('utf-8')
        keyword = keyword.lower()
        chars = keyword.strip()
        if not chars:
            return
        level = self.keyword_chains
        for i in range(len(chars)):
            if chars[i] in level:
                level = level[chars[i]]
            else:
                if not isinstance(level, dict):
                    break
                for j in range(i, len(chars)):
                    level[chars[j]] = {}
                    last_level, last_char = level, chars[j]
                    level = level[chars[j]]
                last_level[last_char] = {self.delimit: 0}
                break
        if i == len(chars) - 1:
            level[self.delimit] = 0

    def parse(self):
        with open(self.path, encoding='utf-8') as f:
            for keyword in f:
                self.add(keyword.strip())

    def word_replace(self, message, repl="*"):
        if isinstance(message, bytes):
            message = message.decode('utf-8')
        message = message.lower()
        ret = []
        start = 0
        while start < len(message):
            level = self.keyword_chains
            step_ins = 0
            for char in message[start:]:
                if char in level:
                    step_ins += 1
                    if self.delimit not in level[char]:
                        level = level[char]
                    else:
                        ret.append(repl * step_ins)
                        start += step_ins - 1
                        break
                else:
                    ret.append(message[start])
                    break
            else:
                ret.append(message[start])
            start += 1

        return ''.join(ret)



"""
    作者：mldsh
    日期：2022年09月21日16:07
    使用工具：PyCharm
"""
import re

import requests
from toollib.snowflake import SnowFlake
from word_filter import DFAFilter
import json


class SendText:
    """
    1.需要一个send text 方法，
    2.判断参数

    """

    def __init__(self, text, wx_id, send_url):
        """

        :param text: 文本
        :param wx_id: 接收人
        :param send_url: 后端服务
        """

        print(text,wx_id,send_url)
        if not isinstance(text, (str, type(None))):
            raise TypeError('"text" only supported: str')
        if not isinstance(wx_id, (str, type(None))):
            raise TypeError('"wx_id" only supported: str')
        if not isinstance(send_url, (str, type(None))):
            raise TypeError('"send_url" only supported: str')
        if self._re_send_url(send_url):
            raise TypeError(
                "send_ The url is required to be a url and must start with https or http. Complete connection")
        if self._check_text(text):
            raise TypeError("Message non-compliance")
        self.snow = SnowFlake()
        self.uid = self.snow.guid()
        self.send_url = send_url
        self.wx_id = wx_id
        self.text = text
        self.headers = {'Content-Type': 'application/json;charset=UTF-8'}


    @staticmethod
    def from_dict(obj, algorithm=None):
        return SendText(obj, algorithm)

    @staticmethod
    def from_json(data, algorithm=None):
        obj = json.loads(data)
        return SendText.from_dict(obj, algorithm)

    def _check_text(self,text):
        """
        验证发送文本消息，是否正常。
        :return: bool
        """
        gfw = DFAFilter()
        result = gfw.word_replace(text)
        if "*" in result:
            return True

    def _re_send_url(self, send_url):
        """
        验证 url 的是否合格
        :param self:
        :param send_url:
        :return: bool
        """
        result = re.search('(https?|ftp|file)://[-A-Za-z0-9+&@#/%?=~_|!:,.;]+[-A-Za-z0-9+&@#/%=~_|]', send_url).group()
        print('222',result)
        if result:
            return False
        else:
            return True

    def bot_send_text(self,text,wx_id, send_url):
        """
        实现调用服务端接口，发送请求。
        :return:
        """
        super.__init__(text, wx_id, send_url)
        json_data = {
            "text": self.text,
            "snow_id": self.uid,
            "wx_id": self.wx_id
        }
        result = requests.post(url=self.send_url, headers=self.headers, json=json_data).json()
        return result




